import unittest
import pycats
import pandas as pd

class TestAsCat(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.x = pd.DataFrame({ 
            'a': [4,1,9,6,2,3,5,7,2,9],
            'b': ['foo', 'foo', 'foo', 'foo', 'foo', 'bar', 'bar', 'bar', 'baz', 'baz2']
        })

    def test_converts_to_cat(self):
        x_cat = pycats.as_cat(self.x['b'])
        self.assertTrue(x_cat.dtype.name == 'category')

    def test_cats_stay_cats(self):
        x_cat = pycats.as_cat(self.x['b'])
        x_cat = pycats.as_cat(x_cat)
        self.assertTrue(x_cat.dtype.name == 'category')

if __name__ == '__main__':
    unittest.main()



__all__ = ['cat_anon', 'cat_lump', 'as_cat', 'cat_collapse',
           'cat_other']

import pandas as pd
import numpy as np


# TODO: add support for ties method
def cat_lump(f, n=None, other_level='Other', ties_method='first'):
    ties_methods = ['min', 'average', 'first']
    if ties_method not in ties_methods:
        raise ValueError('''ties_method not supported. Supported ties_methods are
            ["min", "average", "first']''')

    levels = f.cat.categories
    counts = f.value_counts()
    
    if n is not None and 0 < n < len(counts):
        lump_cutoff = counts[n]
        f = f.apply(lambda row: row if not in_smallest(row, counts.to_dict(), lump_cutoff, ties_method) else other_level)
    elif n is not None and (-1*len(counts)) < n < 0:
        lump_cutoff = counts[n]
        f = f.apply(lambda row: row if in_smallest(row, counts.to_dict(), lump_cutoff, ties_method) else other_level)
    return f.astype('category')


# Given vector of counts, returns logical vector if in
# smallest groups
def in_smallest(level, counts, cutoff, ties_method):
    return counts[level] <= cutoff


def cat_collapse(f, groups):
    for key in groups:
        f = f.apply(lambda row: key if row in groups[key] else row)
    return f.astype('category')


def as_cat(x):
    return x.astype('category')


def cat_anon(f):
    category_count = len(f.cat.categories)
    anon_levels = np.random.choice(10000000, category_count, replace=False)
    return f.cat.rename_categories(anon_levels)


def cat_other(f, drop, other_level = 'Other'):
    f = f.apply(lambda row: row if row not in drop else other_level)
    return f.astype('category')

def cat_drop(f, in_place=False):
    return f.cat.remove_unused_categories(in_place)



import unittest
import pycats
import pandas as pd
import collections

class TestCatOther(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.x = pd.DataFrame({ 
            'a': [4,1,9,6,2,3,5,7,2,9], 
            'b': ['foo', 'foo', 'foo', 'foo', 'foo', 'bar', 'bar', 'bar', 'baz', 'baz2']
        })

    def test_other(self):
        other_cats = pycats.cat_other(self.x['b'], ['foo', 'baz'])
        remaining_cats = ['bar', 'baz2', 'Other']
        self.assertTrue(collections.Counter(other_cats.cat.categories) == collections.Counter(remaining_cats))


if __name__ == '__main__':
    unittest.main()



import unittest
import pycats
import pandas as pd

class TestCatAnon(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.x = pd.DataFrame({ 
            'a': [4,1,9,6,2,3,5,7,2,9],
            'b': ['foo', 'foo', 'foo', 'foo', 'foo', 'bar', 'bar', 'bar', 'baz', 'baz2']
        })

    def test_convert_to_anon(self):
        x_cat = self.x['b'].astype('category')
        anon_cats = pycats.cat_anon(x_cat)
        self.assertTrue(type(anon_cats.iloc[0].item()) is int)

    def test_ints_converted(self):
        x_cat = self.x['b'].astype('category')
        anon_cats = pycats.as_cat(self.x['a'])
        self.assertTrue(type(anon_cats.iloc[0].item()) is int)

if __name__ == '__main__':
    unittest.main()



import unittest
import pycats
import pandas as pd
import collections


class TestCatCollapse(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.x = pd.DataFrame({ 
        'a': [4,1,9,6,3,2,3,2,9],
        'b': ['foo', 'foo', 'foo', 'foo2', 'bar', 'bar', 'bar2', 'baz', 'baz2']
        })

        self.groups = {
            'collapse_bar': ['bar', 'bar2'],
            'collapse_foo': ['foo','foo2']
        }

    def test_collapse(self):
        x_cat = self.x['b'].astype('category')
        collapsed_cats = pycats.cat_collapse(x_cat, self.groups)
        remaining_cats = ['collapse_bar', 'collapse_foo', 'baz', 'baz2']
        self.assertTrue(collections.Counter(collapsed_cats.cat.categories) == collections.Counter(remaining_cats))

if __name__ == '__main__':
    unittest.main()



import unittest
import pycats
import pandas as pd
import collections

class TestCatLump(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.x = pd.DataFrame({ 
            'a': [4,1,9,6,2,3,5,7,2,9], 
            'b': ['foo', 'foo', 'foo', 'foo', 'foo', 'bar', 'bar', 'bar', 'baz', 'baz2']
        })

    def test_lump(self):
        as_cat = self.x['b'].astype('category')
        lump_cats = pycats.cat_lump(as_cat, 2)
        remaining_cats = ['foo', 'bar', 'Other']
        self.assertTrue(collections.Counter(lump_cats.cat.categories) == collections.Counter(remaining_cats))


if __name__ == '__main__':
    unittest.main()
